import { API } from "./Constants"
import { getSession, saveSession, Session } from "./api/session.ts"

export async function fetchAPI(
    url: string,
    payload: unknown,
    method = "POST",
): Promise<Response> {
    const session = getSession()
    const token = session?.auth?.token

    const headers: HeadersInit = {
        Accept: "application/json",
        "Content-Type": "application/json",
    }

    if (token) {
        headers.Authorization = token
    }

    const response = await fetch(`${API}/${url}`, {
        method,
        headers,
        body: JSON.stringify(payload),
    })

    return await handleResponse(session, response)
}

export async function fetchAPIGet(url: string): Promise<Response> {
    const session = getSession()
    const token = session?.auth?.token

    const headers: HeadersInit = {
        Accept: "application/json",
        "Content-Type": "application/json",
    }

    if (token) {
        headers.Authorization = token
    }

    const response = await fetch(`${API}/${url}`, {
        method: "GET",
        headers,
    })

    return await handleResponse(session, response)
}

async function handleResponse(
    session: Session,
    response: Response,
): Promise<Response> {
    // if we provided a token but still unauthorized, the token has expired
    if (!response.ok) {
        return response
    }

    const nextToken = response.headers.get("Next-Authorization")!
    const expirationDate = Date.parse(
        response.headers.get("Next-Authorization-Expiration-Date")!,
    )
    if (nextToken && expirationDate)
        saveSession({ ...session, auth: { token: nextToken, expirationDate } })

    return response
}
